#!/usr/bin/env python2
# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c), The AiiDA-CP2K authors.                                      #
# SPDX-License-Identifier: MIT                                                #
# AiiDA-CP2K is hosted on GitHub at https://github.com/cp2k/aiida-cp2k        #
# For further information on the license, see the LICENSE.txt file.           #
###############################################################################

from __future__ import print_function

import sys
import ase.build
import numpy as np
from utils import wait_for_calc

from aiida import load_dbenv, is_dbenv_loaded
from aiida.backends import settings
if not is_dbenv_loaded():
    load_dbenv(profile=settings.AIIDADB_PROFILE)

from aiida.common.example_helpers import test_and_get_code  # noqa
from aiida.orm.data.structure import StructureData  # noqa
from aiida.orm.data.parameter import ParameterData  # noqa
from aiida.orm.data.singlefile import SinglefileData  # noqa


# ==============================================================================
if len(sys.argv) != 2:
    print("Usage: test_precision.py <code_name>")
    sys.exit(1)

codename = sys.argv[1]
code = test_and_get_code(codename, expected_code_type='cp2k')

print("Testing structure roundtrip precision ase->aiida->cp2k->aiida->ase...")

# calc object
calc = code.new_calc()

# structure
epsilon = 1e-10  # expected precision in Angstrom
dist = 0.74 + epsilon
positions = [(0, 0, 0), (0, 0, dist)]
cell = np.diag([4, -4, 4 + epsilon])
atoms = ase.Atoms('H2', positions=positions, cell=cell)
structure = StructureData(ase=atoms)
calc.use_structure(structure)


# parameters
parameters = ParameterData(dict={
    'GLOBAL': {
        'RUN_TYPE': 'MD',
    },
    'MOTION': {
        'MD': {
            'TIMESTEP': 0.0,  # do not move atoms
            'STEPS': 1,
        },
    },
    'FORCE_EVAL': {
        'METHOD': 'Quickstep',
        'DFT': {
            'BASIS_SET_FILE_NAME': 'BASIS_MOLOPT',
            'SCF': {
                 'MAX_SCF': 1,
            },
            'XC': {
                'XC_FUNCTIONAL': {
                    '_': 'LDA',
                },
            },
        },
        'SUBSYS': {
            'KIND': {
                '_': 'DEFAULT',
                'BASIS_SET': 'DZVP-MOLOPT-SR-GTH',
                'POTENTIAL': 'GTH-LDA',
            },
        },
    },
})
calc.use_parameters(parameters)

# resources
calc.set_max_wallclock_seconds(3*60)  # 3 min
calc.set_resources({"num_machines": 1})

# store and submit
calc.store_all()
calc.submit()
print("submitted calculation: PK=%s" % calc.pk)

wait_for_calc(calc)

# check structure preservation
atoms2 = calc.out.output_structure.get_ase()

# zeros should be preserved exactly
if np.all(atoms2.positions[0] == 0.0):
    print("OK, zeros in structure were preserved exactly")
else:
    print("ERROR!")
    print("Zeros in structure changed: ", atoms2.positions[0])
    sys.exit(3)

# other values should be preserved with epsilon precision
dist2 = atoms2.get_distance(0, 1)
if abs(dist2 - dist) < epsilon:
    print("OK, structure preserved with %.1e Angstrom precision" % epsilon)
else:
    print("ERROR!")
    print("Structure changed by %e Angstrom" % abs(dist - dist2))
    sys.exit(3)

# check cell preservation
cell_diff = np.amax(np.abs(atoms2.cell - cell))
if cell_diff < epsilon:
    print("OK, cell preserved with %.1e Angstrom precision" % epsilon)
else:
    print("ERROR!")
    print("Cell changed by %e Angstrom" % cell_diff)
    sys.exit(3)

sys.exit(0)

# EOF



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c), The AiiDA-CP2K authors.                                      #
# SPDX-License-Identifier: MIT                                                #
# AiiDA-CP2K is hosted on GitHub at https://github.com/cp2k/aiida-cp2k        #
# For further information on the license, see the LICENSE.txt file.           #
###############################################################################

from __future__ import print_function

import re
import sys
import ase.build
from copy import deepcopy
from utils import wait_for_calc

from aiida import load_dbenv, is_dbenv_loaded
from aiida.backends import settings
if not is_dbenv_loaded():
    load_dbenv(profile=settings.AIIDADB_PROFILE)

from aiida.orm import load_node  # noqa
from aiida.orm.data.remote import RemoteData  # noqa
from aiida.common.example_helpers import test_and_get_code  # noqa
from aiida.orm.data.structure import StructureData  # noqa
from aiida.orm.data.parameter import ParameterData  # noqa
from aiida.orm.data.singlefile import SinglefileData  # noqa

# ==============================================================================
if len(sys.argv) != 2:
    print("Usage: test_restart.py <code_name>")
    sys.exit(1)

codename = sys.argv[1]
code = test_and_get_code(codename, expected_code_type='cp2k')

print("Testing CP2K restart...")

# structure
atoms1 = ase.build.molecule('H2O')
atoms1.center(vacuum=2.0)
structure1 = StructureData(ase=atoms1)

# CP2K input
params1 = {
    'GLOBAL': {
        'RUN_TYPE': 'GEO_OPT',
        'WALLTIME': '00:00:10',  # too short
    },
    'MOTION': {
        'GEO_OPT': {
            'MAX_FORCE': 1e-20,  # impossible to reach
            'MAX_ITER': 100000  # run forever
        },
    },
    'FORCE_EVAL': {
        'METHOD': 'Quickstep',
        'DFT': {
            'BASIS_SET_FILE_NAME': 'BASIS_MOLOPT',
            'QS': {
                'EPS_DEFAULT': 1.0e-12,
                'WF_INTERPOLATION': 'ps',
                'EXTRAPOLATION_ORDER': 3,
            },
            'MGRID': {
                'NGRIDS': 4,
                'CUTOFF': 280,
                'REL_CUTOFF': 30,
            },
            'XC': {
                'XC_FUNCTIONAL': {
                    '_': 'LDA',
                },
            },
            'POISSON': {
                'PERIODIC': 'none',
                'PSOLVER': 'MT',
            },
            'SCF': {
                'PRINT': {'RESTART': {'_': 'ON'}}
            },
        },
        'SUBSYS': {
            'KIND': [
                {'_': 'O', 'BASIS_SET': 'DZVP-MOLOPT-SR-GTH',
                    'POTENTIAL': 'GTH-LDA-q6'},
                {'_': 'H', 'BASIS_SET': 'DZVP-MOLOPT-SR-GTH',
                    'POTENTIAL': 'GTH-LDA-q1'},
            ],
        },
    }
}

# ------------------------------------------------------------------------------
# Set up the first calculation

calc1 = code.new_calc()
calc1.use_structure(structure1)
calc1.use_parameters(ParameterData(dict=params1))
calc1.set_max_wallclock_seconds(2*60)
calc1.set_resources({"num_machines": 1})
calc1.store_all()
calc1.submit()
print("submitted calculation 1: PK=%s" % calc1.pk)
wait_for_calc(calc1)

# check walltime exceeded
assert calc1.res.exceeded_walltime is True
assert calc1.res.energy is not None
assert calc1.out.output_structure is not None
print("OK, walltime exceeded as expected")

# ------------------------------------------------------------------------------
# Set up and start the second calculation

# parameters
params2 = deepcopy(calc1.inp.parameters.get_dict())
del(params2['GLOBAL']['WALLTIME'])
del(params2['MOTION']['GEO_OPT']['MAX_FORCE'])
restart_wfn_fn = './parent_calc/aiida-RESTART.wfn'
params2['FORCE_EVAL']['DFT']['RESTART_FILE_NAME'] = restart_wfn_fn
params2['FORCE_EVAL']['DFT']['SCF']['SCF_GUESS'] = 'RESTART'
params2['EXT_RESTART'] = {'RESTART_FILE_NAME': './parent_calc/aiida-1.restart'}

# structure
atoms2 = ase.build.molecule('H2O')
atoms2.center(vacuum=2.0)
atoms2.positions *= 0.0  # place all atoms at origin -> nuclear fusion :-)
structure2 = StructureData(ase=atoms2)

calc2 = code.new_calc()
calc2.use_structure(structure2)  # breaks simulation unless overwritten ...
calc2.use_parameters(ParameterData(dict=params2))  # ... by EXT_RESTART
calc2.use_parent_folder(calc1.out.remote_folder)
calc2.set_max_wallclock_seconds(2*60)
calc2.set_resources({"num_machines": 1})
calc2.store_all()
calc2.submit()
print("submitted calculation 2: PK=%s" % calc2.pk)
wait_for_calc(calc2)

# check energy
expected_energy = -17.1566455959
if abs(calc2.res.energy - expected_energy) < 1e-10:
    print("OK, energy has the expected value")

# if restart wfn is not found it will create a warning
assert calc2.res.nwarnings == 1

# ensure that this warning originates from overwritting coordinates
out_fn = calc2.out.retrieved.get_abs_path("aiida.out")
output = open(out_fn).read()
assert re.search("WARNING .* :: Overwriting coordinates", output)

sys.exit(0)
# EOF



# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c), The AiiDA-CP2K authors.                                      #
# SPDX-License-Identifier: MIT                                                #
# AiiDA-CP2K is hosted on GitHub at https://github.com/cp2k/aiida-cp2k        #
# For further information on the license, see the LICENSE.txt file.           #
###############################################################################

import re
import ase
import numpy as np
from re import DOTALL

from aiida.parsers.parser import Parser
from aiida.orm.data.parameter import ParameterData
from aiida.orm.data.structure import StructureData
from aiida.parsers.exceptions import OutputParsingError
from aiida_cp2k.calculations import Cp2kCalculation


class Cp2kParser(Parser):
    """
    Parser for the output of CP2K.
    """

    # --------------------------------------------------------------------------
    def __init__(self, calc):
        """
        Initialize the instance of Cp2kParser
        """
        super(Cp2kParser, self).__init__(calc)

        # check for valid input
        if not isinstance(calc, Cp2kCalculation):
            raise OutputParsingError("Input calc must be a Cp2kCalculation")

    # --------------------------------------------------------------------------
    def parse_with_retrieved(self, retrieved):
        """
        Receives in input a dictionary of retrieved nodes.
        Does all the logic here.
        """
        out_folder = retrieved['retrieved']

        new_nodes_list = []
        self._parse_stdout(out_folder, new_nodes_list)
        try:
            self._parse_trajectory(out_folder, new_nodes_list)
        except Exception:
            pass

        return True, new_nodes_list

    # --------------------------------------------------------------------------
    def _parse_stdout(self, out_folder, new_nodes_list):
        fn = self._calc._OUTPUT_FILE_NAME
        if fn not in out_folder.get_folder_list():
            raise OutputParsingError("Cp2k output file not retrieved")

        result_dict = {'exceeded_walltime': False}
        abs_fn = out_folder.get_abs_path(fn)
        with open(abs_fn, "r") as f:
            for line in f.readlines():
                if line.startswith(' ENERGY| '):
                    result_dict['energy'] = float(line.split()[8])
                    result_dict['energy_units'] = "a.u."
                if 'The number of warnings for this run is' in line:
                    result_dict['nwarnings'] = int(line.split()[-1])
                if 'exceeded requested execution time' in line:
                    result_dict['exceeded_walltime'] = True

        if 'nwarnings' not in result_dict:
            raise OutputParsingError("CP2K did not finish properly.")

        pair = ('output_parameters', ParameterData(dict=result_dict))
        new_nodes_list.append(pair)

    # --------------------------------------------------------------------------
    def _parse_trajectory(self, out_folder, new_nodes_list):
        fn = self._calc._RESTART_FILE_NAME
        if fn not in out_folder.get_folder_list():
            return  # not every run type produces a trajectory

        # read restart file
        abs_fn = out_folder.get_abs_path(fn)
        content = open(abs_fn).read()

        # parse coordinate section
        m = re.search('\n\s*&COORD\n(.*?)\n\s*&END COORD\n', content, DOTALL)
        coord_lines = [line.strip().split() for line in m.group(1).split("\n")]
        symbols = [line[0] for line in coord_lines]
        positions_str = [line[1:] for line in coord_lines]
        positions = np.array(positions_str, np.float64)

        # parse cell section
        m = re.search('\n\s*&CELL\n(.*?)\n\s*&END CELL\n', content, re.DOTALL)
        cell_lines = [line.strip().split() for line in m.group(1).split("\n")]
        cell_str = [line[1:] for line in cell_lines if line[0] in 'ABC']
        cell = np.array(cell_str, np.float64)

        # create StructureData
        atoms = ase.Atoms(symbols=symbols, positions=positions, cell=cell)
        pair = ('output_structure', StructureData(ase=atoms))
        new_nodes_list.append(pair)

# EOF



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c), The AiiDA-CP2K authors.                                      #
# SPDX-License-Identifier: MIT                                                #
# AiiDA-CP2K is hosted on GitHub at https://github.com/cp2k/aiida-cp2k        #
# For further information on the license, see the LICENSE.txt file.           #
###############################################################################

from __future__ import print_function

import sys
from utils import wait_for_calc

from aiida import load_dbenv, is_dbenv_loaded
from aiida.backends import settings
if not is_dbenv_loaded():
    load_dbenv(profile=settings.AIIDADB_PROFILE)

from aiida.common.example_helpers import test_and_get_code  # noqa
from aiida.orm.data.parameter import ParameterData  # noqa


# ==============================================================================
if len(sys.argv) != 2:
    print("Usage: test_failure.py <code_name>")
    sys.exit(1)

codename = sys.argv[1]
code = test_and_get_code(codename, expected_code_type='cp2k')

print("Testing CP2K failure...")

# a broken CP2K input
params = {'GLOBAL': {'FOO_BAR_QUUX': 42}}

calc = code.new_calc()
calc.use_parameters(ParameterData(dict=params))
calc.set_max_wallclock_seconds(2*60)
calc.set_resources({"num_machines": 1})
calc.store_all()
calc.submit()
print("submitted calculation: PK=%s" % calc.pk)

wait_for_calc(calc, ensure_finished_ok=False)

if calc.has_failed():
    print("CP2K failure correctly recognized")
else:
    print("ERROR!")
    print("CP2K failure was not recognized")
    sys.exit(3)

sys.exit(0)
# EOF



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c), The AiiDA-CP2K authors.                                      #
# SPDX-License-Identifier: MIT                                                #
# AiiDA-CP2K is hosted on GitHub at https://github.com/cp2k/aiida-cp2k        #
# For further information on the license, see the LICENSE.txt file.           #
###############################################################################

from __future__ import print_function

import os
import sys
import json

a = os.getenv("TRAVIS_TAG")
b = "v" + json.load(open("setup.json"))['version']

if not a:
    print("TRAVIS_TAG not set")

elif a != b:
    print("ERROR!")
    print("TRAVIS_TAG and version are not consistend: '%s' vs '%s'" % (a, b))
    sys.exit(3)

sys.exit(0)

# EOF



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c), The AiiDA-CP2K authors.                                      #
# SPDX-License-Identifier: MIT                                                #
# AiiDA-CP2K is hosted on GitHub at https://github.com/cp2k/aiida-cp2k        #
# For further information on the license, see the LICENSE.txt file.           #
###############################################################################

from __future__ import print_function

import sys
import ase.build
from utils import wait_for_calc

from aiida import load_dbenv, is_dbenv_loaded
from aiida.backends import settings
if not is_dbenv_loaded():
    load_dbenv(profile=settings.AIIDADB_PROFILE)

from aiida.common.example_helpers import test_and_get_code  # noqa
from aiida.orm.data.structure import StructureData  # noqa
from aiida.orm.data.parameter import ParameterData  # noqa
from aiida.orm.data.singlefile import SinglefileData  # noqa


# ==============================================================================
if len(sys.argv) != 2:
    print("Usage: test_mm.py <code_name>")
    sys.exit(1)

codename = sys.argv[1]
code = test_and_get_code(codename, expected_code_type='cp2k')

print("Testing CP2K ENERGY on H2O (MM) ...")

# calc object
calc = code.new_calc()

# force field
with open("/tmp/water.pot", "w") as f:
    f.write("""BONDS
H    H       0.000     1.5139
O    H     450.000     0.9572

ANGLES
H    O    H      55.000   104.5200

DIHEDRALS

IMPROPER

NONBONDED
H      0.000000  -0.046000     0.224500
O      0.000000  -0.152100     1.768200

HBOND CUTHB 0.5

END""")
water_pot = SinglefileData(file="/tmp/water.pot")
calc.use_file(water_pot, linkname="water_pot")

# structure using pdb format, because it also carries topology information
atoms = ase.build.molecule('H2O')
atoms.center(vacuum=10.0)
atoms.write("/tmp/coords.pdb", format="proteindatabank")
cell = atoms.cell
coords_pdb = SinglefileData(file="/tmp/coords.pdb")
calc.use_file(coords_pdb, linkname="coords_pdb")

# parameters
# based on cp2k/tests/Fist/regtest-1-1/water_1.inp
parameters = ParameterData(dict={
        'FORCE_EVAL': {
            'METHOD': 'fist',
            'MM': {
                'FORCEFIELD': {
                    'PARM_FILE_NAME': 'water.pot',
                    'PARMTYPE': 'CHM',
                    'CHARGE': [
                        {'ATOM': 'O', 'CHARGE': -0.8476},
                        {'ATOM': 'H', 'CHARGE': 0.4238}]
                },
                'POISSON': {'EWALD': {
                    'EWALD_TYPE': 'spme',
                    'ALPHA': 0.44,
                    'GMAX': 24,
                    'O_SPLINE': 6
                }}
            },
            'SUBSYS': {
                'CELL': {
                    'ABC': '%f  %f  %f' % tuple(atoms.cell.diagonal()),
                },
                'TOPOLOGY': {
                    'COORD_FILE_NAME': 'coords.pdb',
                    'COORD_FILE_FORMAT': 'PDB',
                },
            },
        },
        'GLOBAL': {
            'CALLGRAPH': 'master',
            'CALLGRAPH_FILE_NAME': 'runtime'
        }
})
calc.use_parameters(parameters)

# settings
settings_dict = {'additional_retrieve_list': ["runtime.callgraph"]}
settings = ParameterData(dict=settings_dict)
calc.use_settings(settings)

# resources
calc.set_max_wallclock_seconds(3*60)  # 3 min
calc.set_resources({"num_machines": 1})

# store and submit
calc.store_all()
calc.submit()
print("submitted calculation: PK=%s" % calc.pk)

wait_for_calc(calc)

# check warnings
assert calc.res.nwarnings == 0

# check energy
expected_energy = 0.146927412614e-3
if abs(calc.res.energy - expected_energy) < 1e-10:
    print("OK, energy has the expected value")
else:
    print("ERROR!")
    print("Expected energy value: {}".format(expected_energy))
    print("Actual energy value: {}".format(calc.res.energy))
    sys.exit(3)

# check if callgraph is there
if "runtime.callgraph" in calc.out.retrieved.get_folder_list():
    print("OK, callgraph file was retrived")
else:
    print("ERROR!")
    print("Callgraph file was not retrieved.")
    sys.exit(3)

sys.exit(0)

# EOF



# -*- coding: utf-8 -*-
###############################################################################
# Copyright (c), The AiiDA-CP2K authors.                                      #
# SPDX-License-Identifier: MIT                                                #
# AiiDA-CP2K is hosted on GitHub at https://github.com/cp2k/aiida-cp2k        #
# For further information on the license, see the LICENSE.txt file.           #
###############################################################################

from setuptools import setup, find_packages
import json


if __name__ == '__main__':
    with open('setup.json', 'r') as info:
        kwargs = json.load(info)
    setup(
        include_package_data=True,
        packages=find_packages(),
        **kwargs
    )
